@@ -1,6 +1,9 @@
 import re
 import sys
+import string
 import subprocess
+import venv
+from tempfile import TemporaryDirectory
 
 from path import Path
 
@@ -64,6 +67,16 @@ def rewrite_importlib_resources(pkg_files, new_root):
         file.write_text(text)
 
 
+def rewrite_importlib_metadata(pkg_files, new_root):
+    """
+    Rewrite imports in importlib_metadata to redirect to vendored copies.
+    """
+    for file in pkg_files.glob('*.py'):
+        text = file.read_text().replace('typing_extensions', '..typing_extensions')
+        text = text.replace('import zipp', 'from .. import zipp')
+        file.write_text(text)
+
+
 def rewrite_more_itertools(pkg_files: Path):
     """
     Defer import of concurrent.futures. Workaround for #3090.
@@ -79,6 +92,16 @@ def rewrite_more_itertools(pkg_files: Path):
     more_file.write_text(text)
 
 
+def rewrite_nspektr(pkg_files: Path, new_root):
+    for file in pkg_files.glob('*.py'):
+        text = file.read_text()
+        text = re.sub(r' (more_itertools)', rf' {new_root}.\1', text)
+        text = re.sub(r' (jaraco\.\w+)', rf' {new_root}.\1', text)
+        text = re.sub(r' (packaging)', rf' {new_root}.\1', text)
+        text = re.sub(r' (importlib_metadata)', rf' {new_root}.\1', text)
+        file.write_text(text)
+
+
 def clean(vendor):
     """
     Remove all files out of the vendor directory except the meta
@@ -117,11 +140,48 @@ def update_pkg_resources():
 def update_setuptools():
     vendor = Path('setuptools/_vendor')
     install(vendor)
+    install_validate_pyproject(vendor)
     rewrite_packaging(vendor / 'packaging', 'setuptools.extern')
     rewrite_jaraco_text(vendor / 'jaraco/text', 'setuptools.extern')
     rewrite_jaraco(vendor / 'jaraco', 'setuptools.extern')
     rewrite_importlib_resources(vendor / 'importlib_resources', 'setuptools.extern')
+    rewrite_importlib_metadata(vendor / 'importlib_metadata', 'setuptools.extern')
     rewrite_more_itertools(vendor / "more_itertools")
+    rewrite_nspektr(vendor / "nspektr", 'setuptools.extern')
+
+
+def install_validate_pyproject(vendor):
+    """``validate-pyproject`` can be vendorized to remove all dependencies"""
+    req = next(
+        (x for x in (vendor / "vendored.txt").lines() if 'validate-pyproject' in x),
+        "validate-pyproject[all]"
+    )
+
+    pkg, _, _ = req.strip(string.whitespace + "#").partition("#")
+    pkg = pkg.strip()
+
+    opts = {}
+    if sys.version_info[:2] >= (3, 10):
+        opts["ignore_cleanup_errors"] = True
+
+    with TemporaryDirectory(**opts) as tmp:
+        env_builder = venv.EnvBuilder(with_pip=True)
+        env_builder.create(tmp)
+        context = env_builder.ensure_directories(tmp)
+        venv_python = getattr(context, 'env_exec_cmd', context.env_exe)
+
+        subprocess.check_call([venv_python, "-m", "pip", "install", pkg])
+        cmd = [
+            venv_python,
+            "-m",
+            "validate_pyproject.vendoring",
+            f"--output-dir={vendor / '_validate_pyproject' !s}",
+            "--enable-plugins",
+            "setuptools",
+            "distutils",
+            "--very-verbose"
+        ]
+        subprocess.check_call(cmd)
 
 
 __name__ == '__main__' and update_vendored()
